Attribute VB_Name = "Maze"
Option Explicit

' The maze information.
Private NumRows As Integer
Private NumCols As Integer
Private LegalMove() As Boolean

' The size of a square.
Private Const SQUARE_WID = 20
Private Const SQUARE_HGT = 20

' The player's position.
Private PlayerR As Integer
Private PlayerC As Integer
Private StartR As Integer
Private StartC As Integer

' The end position.
Private RFinish As Integer
Private CFinish As Integer

' The drawing surface.
Private Canvas As Form
Private picPlayer As PictureBox

Private StartTime As Single

Private FileName As String
' Draw the maze.
Private Sub DrawMaze()
Dim r As Integer
Dim c As Integer
Dim clr As Long

    ' Start from scratch.
    Canvas.Cls
    
    For r = 1 To NumRows
        For c = 1 To NumCols
            If LegalMove(r, c) Then
                If r = RFinish And c = CFinish Then
                    clr = vbYellow
                Else
                    clr = vbWhite
                End If
            Else
                clr = RGB(128, 128, 128)
            End If
            Canvas.Line (c * SQUARE_WID, r * SQUARE_HGT)-Step(-SQUARE_WID, -SQUARE_HGT), clr, BF
        Next c
    Next r
End Sub
Public Sub InitializeMaze(ByVal the_canvas As Form, ByVal pic_player As PictureBox, ByVal file_name As String)
    FileName = file_name

    Set Canvas = the_canvas
    Canvas.ScaleMode = vbPixels
    Canvas.AutoRedraw = True
    
    Set picPlayer = pic_player
    picPlayer.Visible = False

    ' Load the maze data.
    LoadMaze file_name
End Sub

' Initialize the maze.
Private Sub LoadMaze(ByVal file_name As String)
Dim fnum As Integer
Dim r As Integer
Dim c As Integer
Dim ch As String
Dim row_info As String

    ' Open the maze file.
    fnum = FreeFile
    Open file_name For Input As #fnum

    ' Read the number of rows and columns.
    Input #fnum, NumRows, NumCols
    ReDim LegalMove(1 To NumRows, 1 To NumCols)
    
    ' Read the data.
    For r = 1 To NumRows
        Line Input #fnum, row_info
        For c = 1 To NumCols
            ch = Mid$(row_info, c, 1)
            LegalMove(r, c) = (ch <> "#")
            If LCase$(ch) = "s" Then
                ' It's the start.
                PlayerR = r
                PlayerC = c
                StartR = r
                StartC = c
            ElseIf LCase$(ch) = "f" Then
                ' It's the finish.
                RFinish = r
                CFinish = c
            End If
        Next c
    Next r

    ' Close the file.
    Close #fnum

    ' Size the form.
    Canvas.Width = Canvas.ScaleX(SQUARE_WID * NumCols, Canvas.ScaleMode, vbTwips) + _
        Canvas.Width - Canvas.ScaleX(Canvas.ScaleWidth, Canvas.ScaleMode, vbTwips)
    Canvas.Height = Canvas.ScaleY(SQUARE_HGT * NumRows, Canvas.ScaleMode, vbTwips) + _
        Canvas.Height - Canvas.ScaleY(Canvas.ScaleHeight, Canvas.ScaleMode, vbTwips)

    ' Draw the maze.
    DrawMaze

    ' Position the player.
    PositionPlayer PlayerR, PlayerC

    ' Save the start time.
    StartTime = Timer
End Sub


Public Sub ProcessMoveKey(KeyCode As Integer)
Dim r As Integer
Dim c As Integer
    
    r = PlayerR
    c = PlayerC
    Select Case KeyCode
        Case vbKeyLeft
            c = PlayerC - 1
        Case vbKeyRight
            c = PlayerC + 1
        Case vbKeyDown
            r = PlayerR + 1
        Case vbKeyUp
            r = PlayerR - 1
        Case Else
            Exit Sub
    End Select

    If LegalMove(r, c) Then PositionPlayer r, c
End Sub
' Draw the player.
Private Sub PositionPlayer(r As Integer, c As Integer)
Dim x As Single
Dim y As Single

    ' Erase the player's old position.
    If PlayerR > 0 Then
        x = (PlayerC - 1) * SQUARE_WID + (SQUARE_WID - picPlayer.Width) / 2
        y = (PlayerR - 1) * SQUARE_HGT + (SQUARE_HGT - picPlayer.Height) / 2
        Canvas.Line (x - 1, y - 1)-Step(picPlayer.Width, picPlayer.Height), vbWhite, BF
    End If

    ' Move the player.
    PlayerR = r
    PlayerC = c

    ' Draw the player.
    x = (c - 1) * SQUARE_WID + (SQUARE_WID - picPlayer.Width) / 2
    y = (r - 1) * SQUARE_HGT + (SQUARE_HGT - picPlayer.Height) / 2
    Canvas.PaintPicture picPlayer.Picture, x, y

    ' See if the player reached the finish.
    If r = RFinish And c = CFinish Then
        If MsgBox("You finished in " & _
            Int(Timer - StartTime) & " seconds." & _
            vbCrLf & "Play again?", vbYesNo, _
            "Congratulations") = vbYes _
        Then
            ' Start over.
            ' Restart the timer. Thanks to
            ' Barry Traver (barry@traver.org).
            StartTime = Timer

            PlayerR = StartR
            PlayerC = StartC
            DrawMaze
            PositionPlayer PlayerR, PlayerC
        Else
            Unload Canvas
        End If
    End If
End Sub

